{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Com
  ( module Data.FpML.V53.Com
  , module Data.FpML.V53.Shared.Option
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Shared.Option
 
-- | The acceptable tolerance in the delivered quantity of a 
--   physical commodity product in terms of a number of units of 
--   that product. 
data AbsoluteTolerance
instance Eq AbsoluteTolerance
instance Show AbsoluteTolerance
instance SchemaType AbsoluteTolerance
 
-- | A scheme defining where bullion is to be delivered for a 
--   Bullion Transaction. 
data BullionDeliveryLocation
data BullionDeliveryLocationAttributes
instance Eq BullionDeliveryLocation
instance Eq BullionDeliveryLocationAttributes
instance Show BullionDeliveryLocation
instance Show BullionDeliveryLocationAttributes
instance SchemaType BullionDeliveryLocation
instance Extension BullionDeliveryLocation Scheme
 
-- | Physically settled leg of a physically settled Bullion 
--   Transaction. 
data BullionPhysicalLeg
instance Eq BullionPhysicalLeg
instance Show BullionPhysicalLeg
instance SchemaType BullionPhysicalLeg
instance Extension BullionPhysicalLeg PhysicalForwardLeg
instance Extension BullionPhysicalLeg CommodityForwardLeg
instance Extension BullionPhysicalLeg Leg
 
-- | A pointer style reference to single-day-duration 
--   calculation periods defined elsewhere - note that this 
--   schedule consists of a parameterised schedule in a 
--   calculationPeriodsSchedule container. 
data CalculationPeriodsDatesReference
instance Eq CalculationPeriodsDatesReference
instance Show CalculationPeriodsDatesReference
instance SchemaType CalculationPeriodsDatesReference
instance Extension CalculationPeriodsDatesReference Reference
 
-- | A pointer style reference to a calculation periods schedule 
--   defined elsewhere - note that this schedule consists of a 
--   series of actual dates in a calculationPeriods container. 
data CalculationPeriodsReference
instance Eq CalculationPeriodsReference
instance Show CalculationPeriodsReference
instance SchemaType CalculationPeriodsReference
instance Extension CalculationPeriodsReference Reference
 
-- | A pointer style reference to a calculation periods schedule 
--   defined elsewhere - note that this schedule consists of a 
--   parameterised schedule in a calculationPeriodsSchedule 
--   container. 
data CalculationPeriodsScheduleReference
instance Eq CalculationPeriodsScheduleReference
instance Show CalculationPeriodsScheduleReference
instance SchemaType CalculationPeriodsScheduleReference
instance Extension CalculationPeriodsScheduleReference Reference
 
-- | The different options for specifying the attributes of a 
--   coal quality measure as a decimal value. 
data CoalAttributeDecimal
instance Eq CoalAttributeDecimal
instance Show CoalAttributeDecimal
instance SchemaType CoalAttributeDecimal
 
-- | The different options for specifying the attributes of a 
--   coal quality measure as a percentage of the measured value. 
data CoalAttributePercentage
instance Eq CoalAttributePercentage
instance Show CoalAttributePercentage
instance SchemaType CoalAttributePercentage
 
-- | The physical delivery conditions for coal. 
data CoalDelivery
instance Eq CoalDelivery
instance Show CoalDelivery
instance SchemaType CoalDelivery
 
-- | A scheme identifying the types of the Delivery Point for a 
--   physically settled coal trade. 
data CoalDeliveryPoint
data CoalDeliveryPointAttributes
instance Eq CoalDeliveryPoint
instance Eq CoalDeliveryPointAttributes
instance Show CoalDeliveryPoint
instance Show CoalDeliveryPointAttributes
instance SchemaType CoalDeliveryPoint
instance Extension CoalDeliveryPoint Scheme
 
-- | Physically settled leg of a physically settled coal 
--   transaction. 
data CoalPhysicalLeg
instance Eq CoalPhysicalLeg
instance Show CoalPhysicalLeg
instance SchemaType CoalPhysicalLeg
instance Extension CoalPhysicalLeg PhysicalSwapLeg
instance Extension CoalPhysicalLeg CommoditySwapLeg
instance Extension CoalPhysicalLeg Leg
 
-- | A type defining the characteristics of the coal being 
--   traded in a physically settled gas transaction. 
data CoalProduct
instance Eq CoalProduct
instance Show CoalProduct
instance SchemaType CoalProduct
 
-- | A scheme identifying the sources of coal for a physically 
--   settled coal trade. 
data CoalProductSource
data CoalProductSourceAttributes
instance Eq CoalProductSource
instance Eq CoalProductSourceAttributes
instance Show CoalProductSource
instance Show CoalProductSourceAttributes
instance SchemaType CoalProductSource
instance Extension CoalProductSource Scheme
 
-- | The different options for specifying the quality attributes 
--   of the coal to be delivered. 
data CoalProductSpecifications
instance Eq CoalProductSpecifications
instance Show CoalProductSpecifications
instance SchemaType CoalProductSpecifications
 
-- | A scheme identifying the types of coal for a physically 
--   settled coal trade. 
data CoalProductType
data CoalProductTypeAttributes
instance Eq CoalProductType
instance Eq CoalProductTypeAttributes
instance Show CoalProductType
instance Show CoalProductTypeAttributes
instance SchemaType CoalProductType
instance Extension CoalProductType Scheme
 
-- | A scheme identifying the quality adjustment formulae for a 
--   physically settled coal trade. 
data CoalQualityAdjustments
data CoalQualityAdjustmentsAttributes
instance Eq CoalQualityAdjustments
instance Eq CoalQualityAdjustmentsAttributes
instance Show CoalQualityAdjustments
instance Show CoalQualityAdjustmentsAttributes
instance SchemaType CoalQualityAdjustments
instance Extension CoalQualityAdjustments Scheme
 
-- | The quality attributes of the coal to be delivered. 
data CoalStandardQuality
instance Eq CoalStandardQuality
instance Show CoalStandardQuality
instance SchemaType CoalStandardQuality
 
-- | The quality attributes of the coal to be delivered, 
--   specified on a periodic basis. 
data CoalStandardQualitySchedule
instance Eq CoalStandardQualitySchedule
instance Show CoalStandardQualitySchedule
instance SchemaType CoalStandardQualitySchedule
 
-- | A scheme identifying the methods by which coal may be 
--   transported. 
data CoalTransportationEquipment
data CoalTransportationEquipmentAttributes
instance Eq CoalTransportationEquipment
instance Eq CoalTransportationEquipmentAttributes
instance Show CoalTransportationEquipment
instance Show CoalTransportationEquipmentAttributes
instance SchemaType CoalTransportationEquipment
instance Extension CoalTransportationEquipment Scheme
 
-- | A type for defining exercise procedures associated with an 
--   American style exercise of a commodity option. 
data CommodityAmericanExercise
instance Eq CommodityAmericanExercise
instance Show CommodityAmericanExercise
instance SchemaType CommodityAmericanExercise
instance Extension CommodityAmericanExercise Exercise
 
-- | A parametric representation of the Calculation Periods for 
--   on Asian option or a leg of a swap. In case the calculation 
--   frequency is of value T (term), the period is defined by 
--   the commoditySwap\effectiveDate and the 
--   commoditySwap\terminationDate. 
data CommodityCalculationPeriodsSchedule
instance Eq CommodityCalculationPeriodsSchedule
instance Show CommodityCalculationPeriodsSchedule
instance SchemaType CommodityCalculationPeriodsSchedule
instance Extension CommodityCalculationPeriodsSchedule Frequency
 
-- | The different options for specifying the Delivery Periods 
--   of a physical leg. 
data CommodityDeliveryPeriods
instance Eq CommodityDeliveryPeriods
instance Show CommodityDeliveryPeriods
instance SchemaType CommodityDeliveryPeriods
 
-- | A scheme identifying the types of the Delivery Point for a 
--   physically settled commodity trade. 
data CommodityDeliveryPoint
data CommodityDeliveryPointAttributes
instance Eq CommodityDeliveryPoint
instance Eq CommodityDeliveryPointAttributes
instance Show CommodityDeliveryPoint
instance Show CommodityDeliveryPointAttributes
instance SchemaType CommodityDeliveryPoint
instance Extension CommodityDeliveryPoint Scheme
 
-- | A scheme identifying how the parties to the trade aportion 
--   responsibility for the delivery of the commodity product 
--   (for example Free On Board, Cost, Insurance, Freight) 
data CommodityDeliveryRisk
data CommodityDeliveryRiskAttributes
instance Eq CommodityDeliveryRisk
instance Eq CommodityDeliveryRiskAttributes
instance Show CommodityDeliveryRisk
instance Show CommodityDeliveryRiskAttributes
instance SchemaType CommodityDeliveryRisk
instance Extension CommodityDeliveryRisk Scheme
 
-- | A type for defining exercise procedures associated with a 
--   European style exercise of a commodity option. 
data CommodityEuropeanExercise
instance Eq CommodityEuropeanExercise
instance Show CommodityEuropeanExercise
instance SchemaType CommodityEuropeanExercise
instance Extension CommodityEuropeanExercise Exercise
 
-- | The parameters for defining how the commodity option can be 
--   exercised, how it is priced and how it is settled. 
data CommodityExercise
instance Eq CommodityExercise
instance Show CommodityExercise
instance SchemaType CommodityExercise
 
data CommodityExercisePeriods
instance Eq CommodityExercisePeriods
instance Show CommodityExercisePeriods
instance SchemaType CommodityExercisePeriods
 
-- | A scheme identifying the physical event relative to which 
--   option expiration occurs. 
data CommodityExpireRelativeToEvent
data CommodityExpireRelativeToEventAttributes
instance Eq CommodityExpireRelativeToEvent
instance Eq CommodityExpireRelativeToEventAttributes
instance Show CommodityExpireRelativeToEvent
instance Show CommodityExpireRelativeToEventAttributes
instance SchemaType CommodityExpireRelativeToEvent
instance Extension CommodityExpireRelativeToEvent Scheme
 
-- | Commodity Forward 
data CommodityForward
instance Eq CommodityForward
instance Show CommodityForward
instance SchemaType CommodityForward
instance Extension CommodityForward Product
 
-- | The Fixed Price for a given Calculation Period during the 
--   life of the trade. There must be a Fixed Price step 
--   specified for each Calculation Period, regardless of 
--   whether the Fixed Price changes or remains the same between 
--   periods. 
data CommodityFixedPriceSchedule
instance Eq CommodityFixedPriceSchedule
instance Show CommodityFixedPriceSchedule
instance SchemaType CommodityFixedPriceSchedule
 
-- | Abstract base class for all commodity forward legs 
data CommodityForwardLeg
instance Eq CommodityForwardLeg
instance Show CommodityForwardLeg
instance SchemaType CommodityForwardLeg
instance Extension CommodityForwardLeg Leg
 
-- | Frequency Type for use in Pricing Date specifications. 
data CommodityFrequencyType
data CommodityFrequencyTypeAttributes
instance Eq CommodityFrequencyType
instance Eq CommodityFrequencyTypeAttributes
instance Show CommodityFrequencyType
instance Show CommodityFrequencyTypeAttributes
instance SchemaType CommodityFrequencyType
instance Extension CommodityFrequencyType Scheme
 
-- | A type defining the FX observations to be used to convert 
--   the observed Commodity Reference Price to the Settlement 
--   Currency. The rate source must be specified. Additionally, 
--   a time for the spot price to be observed on that source may 
--   be specified, or else an averaging schedule for trades 
--   priced using an average FX rate. 
data CommodityFx
instance Eq CommodityFx
instance Show CommodityFx
instance SchemaType CommodityFx
 
-- | Identifes how the FX rate will be applied. This is intended 
--   to differentiate between the various methods for applying 
--   FX to the floating price such as a daily calculation, or 
--   averaging the FX and applying the average at the end of 
--   each CalculationPeriod. 
data CommodityFxType
data CommodityFxTypeAttributes
instance Eq CommodityFxType
instance Eq CommodityFxTypeAttributes
instance Show CommodityFxType
instance Show CommodityFxTypeAttributes
instance SchemaType CommodityFxType
instance Extension CommodityFxType Scheme
 
-- | A type defining a hub or other reference for a physically 
--   settled commodity trade. 
data CommodityHub
instance Eq CommodityHub
instance Show CommodityHub
instance SchemaType CommodityHub
 
-- | A scheme identifying the code for a hub or other reference 
--   for a physically settled commodity trade. 
data CommodityHubCode
data CommodityHubCodeAttributes
instance Eq CommodityHubCode
instance Eq CommodityHubCodeAttributes
instance Show CommodityHubCode
instance Show CommodityHubCodeAttributes
instance SchemaType CommodityHubCode
instance Extension CommodityHubCode Scheme
 
-- | ISDA 1993 or 2005 commodity market disruption elements. 
data CommodityMarketDisruption
instance Eq CommodityMarketDisruption
instance Show CommodityMarketDisruption
instance SchemaType CommodityMarketDisruption
 
-- | A type for defining the multiple exercise provisions of an 
--   American style commodity option. 
data CommodityMultipleExercise
instance Eq CommodityMultipleExercise
instance Show CommodityMultipleExercise
instance SchemaType CommodityMultipleExercise
 
-- | Commodity Notional. 
data CommodityNotionalQuantity
instance Eq CommodityNotionalQuantity
instance Show CommodityNotionalQuantity
instance SchemaType CommodityNotionalQuantity
 
-- | The Notional Quantity per Calculation Period. There must be 
--   a Notional Quantity step specified for each Calculation 
--   Period, regardless of whether the Notional Quantity changes 
--   or remains the same between periods. 
data CommodityNotionalQuantitySchedule
instance Eq CommodityNotionalQuantitySchedule
instance Show CommodityNotionalQuantitySchedule
instance SchemaType CommodityNotionalQuantitySchedule
 
-- | Commodity Option. 
data CommodityOption
instance Eq CommodityOption
instance Show CommodityOption
instance SchemaType CommodityOption
instance Extension CommodityOption Product
 
-- | A scheme identifying the physical event relative to which 
--   payment occurs. 
data CommodityPayRelativeToEvent
data CommodityPayRelativeToEventAttributes
instance Eq CommodityPayRelativeToEvent
instance Eq CommodityPayRelativeToEventAttributes
instance Show CommodityPayRelativeToEvent
instance Show CommodityPayRelativeToEventAttributes
instance SchemaType CommodityPayRelativeToEvent
instance Extension CommodityPayRelativeToEvent Scheme
 
-- | The parameters for defining the expiration date(s) and 
--   time(s) for an American style option. 
data CommodityPhysicalAmericanExercise
instance Eq CommodityPhysicalAmericanExercise
instance Show CommodityPhysicalAmericanExercise
instance SchemaType CommodityPhysicalAmericanExercise
instance Extension CommodityPhysicalAmericanExercise Exercise
 
-- | The parameters for defining the expiration date(s) and 
--   time(s) for a European style option. 
data CommodityPhysicalEuropeanExercise
instance Eq CommodityPhysicalEuropeanExercise
instance Show CommodityPhysicalEuropeanExercise
instance SchemaType CommodityPhysicalEuropeanExercise
instance Extension CommodityPhysicalEuropeanExercise Exercise
 
-- | The parameters for defining how the physically-settled 
--   commodity option can be exercised and how it is settled. 
data CommodityPhysicalExercise
instance Eq CommodityPhysicalExercise
instance Show CommodityPhysicalExercise
instance SchemaType CommodityPhysicalExercise
 
-- | A type defining the physical quantity of the commodity to 
--   be delivered. 
data CommodityPhysicalQuantity
instance Eq CommodityPhysicalQuantity
instance Show CommodityPhysicalQuantity
instance SchemaType CommodityPhysicalQuantity
instance Extension CommodityPhysicalQuantity CommodityPhysicalQuantityBase
 
-- | An abstract base class for physical quantity types. 
data CommodityPhysicalQuantityBase
instance Eq CommodityPhysicalQuantityBase
instance Show CommodityPhysicalQuantityBase
instance SchemaType CommodityPhysicalQuantityBase
 
-- | The Quantity per Delivery Period. There must be a Quantity 
--   step specified for each Delivery Period, regardless of 
--   whether the Quantity changes or remains the same between 
--   periods. 
data CommodityPhysicalQuantitySchedule
instance Eq CommodityPhysicalQuantitySchedule
instance Show CommodityPhysicalQuantitySchedule
instance SchemaType CommodityPhysicalQuantitySchedule
 
-- | The pipeline through which the physical commodity will be 
--   delivered. 
data CommodityPipeline
data CommodityPipelineAttributes
instance Eq CommodityPipeline
instance Eq CommodityPipelineAttributes
instance Show CommodityPipeline
instance Show CommodityPipelineAttributes
instance SchemaType CommodityPipeline
instance Extension CommodityPipeline Scheme
 
-- | The pipeline cycle during which the physical commodity will 
--   be delivered. 
data CommodityPipelineCycle
data CommodityPipelineCycleAttributes
instance Eq CommodityPipelineCycle
instance Eq CommodityPipelineCycleAttributes
instance Show CommodityPipelineCycle
instance Show CommodityPipelineCycleAttributes
instance SchemaType CommodityPipelineCycle
instance Extension CommodityPipelineCycle Scheme
 
-- | The commodity option premium payable by the buyer to the 
--   seller. 
data CommodityPremium
instance Eq CommodityPremium
instance Show CommodityPremium
instance SchemaType CommodityPremium
instance Extension CommodityPremium NonNegativePayment
instance Extension CommodityPremium PaymentBaseExtended
instance Extension CommodityPremium PaymentBase
 
-- | The dates on which prices are observed for the underlyer. 
data CommodityPricingDates
instance Eq CommodityPricingDates
instance Show CommodityPricingDates
instance SchemaType CommodityPricingDates
 
-- | A scheme identifying the grade of physical commodity 
--   product to be delivered. 
data CommodityProductGrade
data CommodityProductGradeAttributes
instance Eq CommodityProductGrade
instance Eq CommodityProductGradeAttributes
instance Show CommodityProductGrade
instance Show CommodityProductGradeAttributes
instance SchemaType CommodityProductGrade
instance Extension CommodityProductGrade Scheme
 
-- | A type for defining the frequency at which the Notional 
--   Quantity is deemed to apply for purposes of calculating the 
--   Total Notional Quantity. 
data CommodityQuantityFrequency
data CommodityQuantityFrequencyAttributes
instance Eq CommodityQuantityFrequency
instance Eq CommodityQuantityFrequencyAttributes
instance Show CommodityQuantityFrequency
instance Show CommodityQuantityFrequencyAttributes
instance SchemaType CommodityQuantityFrequency
instance Extension CommodityQuantityFrequency Scheme
 
-- | The Expiration Dates of the trade relative to the 
--   Calculation Periods. 
data CommodityRelativeExpirationDates
instance Eq CommodityRelativeExpirationDates
instance Show CommodityRelativeExpirationDates
instance SchemaType CommodityRelativeExpirationDates
 
-- | The Payment Dates of the trade relative to the Calculation 
--   Periods. 
data CommodityRelativePaymentDates
instance Eq CommodityRelativePaymentDates
instance Show CommodityRelativePaymentDates
instance SchemaType CommodityRelativePaymentDates
 
-- | The notional quantity of electricity that applies to one or 
--   more groups of Settlement Periods. 
data CommoditySettlementPeriodsNotionalQuantity
instance Eq CommoditySettlementPeriodsNotionalQuantity
instance Show CommoditySettlementPeriodsNotionalQuantity
instance SchemaType CommoditySettlementPeriodsNotionalQuantity
instance Extension CommoditySettlementPeriodsNotionalQuantity CommodityNotionalQuantity
 
-- | The notional quantity schedule of electricity that applies 
--   to one or more groups of Settlement Periods. 
data CommoditySettlementPeriodsNotionalQuantitySchedule
instance Eq CommoditySettlementPeriodsNotionalQuantitySchedule
instance Show CommoditySettlementPeriodsNotionalQuantitySchedule
instance SchemaType CommoditySettlementPeriodsNotionalQuantitySchedule
 
-- | The fixed price schedule for electricity that applies to 
--   one or more groups of Settlement Periods. 
data CommoditySettlementPeriodsPriceSchedule
instance Eq CommoditySettlementPeriodsPriceSchedule
instance Show CommoditySettlementPeriodsPriceSchedule
instance SchemaType CommoditySettlementPeriodsPriceSchedule
 
data CommoditySpread
instance Eq CommoditySpread
instance Show CommoditySpread
instance SchemaType CommoditySpread
instance Extension CommoditySpread Money
instance Extension CommoditySpread MoneyBase
 
-- | The Spread per Calculation Period. There must be a Spread 
--   specified for each Calculation Period, regardless of 
--   whether the Spread changes or remains the same between 
--   periods. 
data CommoditySpreadSchedule
instance Eq CommoditySpreadSchedule
instance Show CommoditySpreadSchedule
instance SchemaType CommoditySpreadSchedule
 
-- | The Strike Price per Unit per Calculation Period. There 
--   must be a Strike Price per Unit step specified for each 
--   Calculation Period, regardless of whether the Strike 
--   changes or remains the same between periods. 
data CommodityStrikeSchedule
instance Eq CommodityStrikeSchedule
instance Show CommodityStrikeSchedule
instance SchemaType CommodityStrikeSchedule
 
-- | Commodity Swap. 
data CommoditySwap
instance Eq CommoditySwap
instance Show CommoditySwap
instance SchemaType CommoditySwap
instance Extension CommoditySwap Product
 
-- | Commodity Swaption. 
data CommoditySwaption
instance Eq CommoditySwaption
instance Show CommoditySwaption
instance SchemaType CommoditySwaption
instance Extension CommoditySwaption Product
 
data CommoditySwaptionUnderlying
instance Eq CommoditySwaptionUnderlying
instance Show CommoditySwaptionUnderlying
instance SchemaType CommoditySwaptionUnderlying
 
-- | Abstract base class for all commodity swap legs 
data CommoditySwapLeg
instance Eq CommoditySwapLeg
instance Show CommoditySwapLeg
instance SchemaType CommoditySwapLeg
instance Extension CommoditySwapLeg Leg
 
-- | A Disruption Fallback. 
data DisruptionFallback
data DisruptionFallbackAttributes
instance Eq DisruptionFallback
instance Eq DisruptionFallbackAttributes
instance Show DisruptionFallback
instance Show DisruptionFallbackAttributes
instance SchemaType DisruptionFallback
instance Extension DisruptionFallback Scheme
 
-- | The physical delivery conditions for electricity. 
data ElectricityDelivery
instance Eq ElectricityDelivery
instance Show ElectricityDelivery
instance SchemaType ElectricityDelivery
 
-- | The physical delivery obligation options specific to a firm 
--   transaction. 
data ElectricityDeliveryFirm
instance Eq ElectricityDeliveryFirm
instance Show ElectricityDeliveryFirm
instance SchemaType ElectricityDeliveryFirm
 
-- | The different options for specifying the Delivery Periods 
--   for a physically settled electricity trade. 
data ElectricityDeliveryPeriods
instance Eq ElectricityDeliveryPeriods
instance Show ElectricityDeliveryPeriods
instance SchemaType ElectricityDeliveryPeriods
instance Extension ElectricityDeliveryPeriods CommodityDeliveryPeriods
 
-- | A scheme identifying the types of the Delivery Point for a 
--   physically settled electricity trade. 
data ElectricityDeliveryPoint
data ElectricityDeliveryPointAttributes
instance Eq ElectricityDeliveryPoint
instance Eq ElectricityDeliveryPointAttributes
instance Show ElectricityDeliveryPoint
instance Show ElectricityDeliveryPointAttributes
instance SchemaType ElectricityDeliveryPoint
instance Extension ElectricityDeliveryPoint Scheme
 
-- | The physical delivery obligation options specific to a 
--   system firm transaction. 
data ElectricityDeliverySystemFirm
instance Eq ElectricityDeliverySystemFirm
instance Show ElectricityDeliverySystemFirm
instance SchemaType ElectricityDeliverySystemFirm
 
data ElectricityDeliveryType
instance Eq ElectricityDeliveryType
instance Show ElectricityDeliveryType
instance SchemaType ElectricityDeliveryType
 
-- | The physical delivery obligation options specific to a unit 
--   firm transaction. 
data ElectricityDeliveryUnitFirm
instance Eq ElectricityDeliveryUnitFirm
instance Show ElectricityDeliveryUnitFirm
instance SchemaType ElectricityDeliveryUnitFirm
 
-- | A type defining the physical quantity of the electricity to 
--   be delivered. 
data ElectricityPhysicalDeliveryQuantity
instance Eq ElectricityPhysicalDeliveryQuantity
instance Show ElectricityPhysicalDeliveryQuantity
instance SchemaType ElectricityPhysicalDeliveryQuantity
instance Extension ElectricityPhysicalDeliveryQuantity CommodityNotionalQuantity
 
-- | Allows the documentation of a shaped quantity trade where 
--   the quantity changes over the life of the transaction. 
data ElectricityPhysicalDeliveryQuantitySchedule
instance Eq ElectricityPhysicalDeliveryQuantitySchedule
instance Show ElectricityPhysicalDeliveryQuantitySchedule
instance SchemaType ElectricityPhysicalDeliveryQuantitySchedule
instance Extension ElectricityPhysicalDeliveryQuantitySchedule CommodityPhysicalQuantitySchedule
 
-- | Physically settled leg of a physically settled electricity 
--   transaction. 
data ElectricityPhysicalLeg
instance Eq ElectricityPhysicalLeg
instance Show ElectricityPhysicalLeg
instance SchemaType ElectricityPhysicalLeg
instance Extension ElectricityPhysicalLeg PhysicalSwapLeg
instance Extension ElectricityPhysicalLeg CommoditySwapLeg
instance Extension ElectricityPhysicalLeg Leg
 
-- | The quantity of gas to be delivered. 
data ElectricityPhysicalQuantity
instance Eq ElectricityPhysicalQuantity
instance Show ElectricityPhysicalQuantity
instance SchemaType ElectricityPhysicalQuantity
instance Extension ElectricityPhysicalQuantity CommodityPhysicalQuantityBase
 
-- | The specification of the electricity to be delivered. 
data ElectricityProduct
instance Eq ElectricityProduct
instance Show ElectricityProduct
instance SchemaType ElectricityProduct
 
-- | A structure to specify the tranmission contingency and the 
--   party that bears the obligation. 
data ElectricityTransmissionContingency
instance Eq ElectricityTransmissionContingency
instance Show ElectricityTransmissionContingency
instance SchemaType ElectricityTransmissionContingency
 
-- | The type of transmission contingency, i.e. what portion of 
--   the transmission the delivery obligations are applicable. 
data ElectricityTransmissionContingencyType
data ElectricityTransmissionContingencyTypeAttributes
instance Eq ElectricityTransmissionContingencyType
instance Eq ElectricityTransmissionContingencyTypeAttributes
instance Show ElectricityTransmissionContingencyType
instance Show ElectricityTransmissionContingencyTypeAttributes
instance SchemaType ElectricityTransmissionContingencyType
instance Extension ElectricityTransmissionContingencyType Scheme
 
-- | The common components of a financially settled leg of a 
--   Commodity Swap. This is an abstract type and should be 
--   extended by commodity-specific types. 
data FinancialSwapLeg
instance Eq FinancialSwapLeg
instance Show FinancialSwapLeg
instance SchemaType FinancialSwapLeg
instance Extension FinancialSwapLeg CommoditySwapLeg
 
-- | A type defining the Fixed Price. 
data FixedPrice
instance Eq FixedPrice
instance Show FixedPrice
instance SchemaType FixedPrice
 
-- | Fixed Price Leg of a Commodity Swap. 
data FixedPriceLeg
instance Eq FixedPriceLeg
instance Show FixedPriceLeg
instance SchemaType FixedPriceLeg
instance Extension FixedPriceLeg FinancialSwapLeg
instance Extension FixedPriceLeg CommoditySwapLeg
instance Extension FixedPriceLeg Leg
 
-- | A type to capture details relevant to the calculation of 
--   the floating price. 
data FloatingLegCalculation
instance Eq FloatingLegCalculation
instance Show FloatingLegCalculation
instance SchemaType FloatingLegCalculation
 
-- | Floating Price Leg of a Commodity Swap. 
data FloatingPriceLeg
instance Eq FloatingPriceLeg
instance Show FloatingPriceLeg
instance SchemaType FloatingPriceLeg
instance Extension FloatingPriceLeg FinancialSwapLeg
instance Extension FloatingPriceLeg CommoditySwapLeg
instance Extension FloatingPriceLeg Leg
 
-- | The specification of the gas to be delivered. 
data GasDelivery
instance Eq GasDelivery
instance Show GasDelivery
instance SchemaType GasDelivery
 
-- | A scheme identifying the types of the Delivery Point for a 
--   physically settled gas trade. 
data GasDeliveryPoint
data GasDeliveryPointAttributes
instance Eq GasDeliveryPoint
instance Eq GasDeliveryPointAttributes
instance Show GasDeliveryPoint
instance Show GasDeliveryPointAttributes
instance SchemaType GasDeliveryPoint
instance Extension GasDeliveryPoint Scheme
 
-- | The different options for specifying the Delivery Periods 
--   for a physically settled gas trade. 
data GasDeliveryPeriods
instance Eq GasDeliveryPeriods
instance Show GasDeliveryPeriods
instance SchemaType GasDeliveryPeriods
instance Extension GasDeliveryPeriods CommodityDeliveryPeriods
 
-- | Physically settled leg of a physically settled gas 
--   transaction. 
data GasPhysicalLeg
instance Eq GasPhysicalLeg
instance Show GasPhysicalLeg
instance SchemaType GasPhysicalLeg
instance Extension GasPhysicalLeg PhysicalSwapLeg
instance Extension GasPhysicalLeg CommoditySwapLeg
instance Extension GasPhysicalLeg Leg
 
-- | The quantity of gas to be delivered. 
data GasPhysicalQuantity
instance Eq GasPhysicalQuantity
instance Show GasPhysicalQuantity
instance SchemaType GasPhysicalQuantity
instance Extension GasPhysicalQuantity CommodityPhysicalQuantityBase
 
-- | A type defining the characteristics of the gas being traded 
--   in a physically settled gas transaction. 
data GasProduct
instance Eq GasProduct
instance Show GasProduct
instance SchemaType GasProduct
 
-- | The quantity of gas to be delivered. 
data GasQuality
data GasQualityAttributes
instance Eq GasQuality
instance Eq GasQualityAttributes
instance Show GasQuality
instance Show GasQualityAttributes
instance SchemaType GasQuality
instance Extension GasQuality Scheme
 
-- | An observation period that is offset from a Calculation 
--   Period. 
data Lag
instance Eq Lag
instance Show Lag
instance SchemaType Lag
 
-- | Allows a lag to reference one already defined elsewhere in 
--   the trade. 
data LagReference
instance Eq LagReference
instance Show LagReference
instance SchemaType LagReference
instance Extension LagReference Reference
 
-- | A Market Disruption Event. 
data MarketDisruptionEvent
data MarketDisruptionEventAttributes
instance Eq MarketDisruptionEvent
instance Eq MarketDisruptionEventAttributes
instance Show MarketDisruptionEvent
instance Show MarketDisruptionEventAttributes
instance SchemaType MarketDisruptionEvent
instance Extension MarketDisruptionEvent Scheme
 
-- | The details of a fixed payment. Can be used for a forward 
--   transaction or as the base for a more complex fixed leg 
--   component such as the fixed leg of a swap. 
data NonPeriodicFixedPriceLeg
instance Eq NonPeriodicFixedPriceLeg
instance Show NonPeriodicFixedPriceLeg
instance SchemaType NonPeriodicFixedPriceLeg
instance Extension NonPeriodicFixedPriceLeg CommoditySwapLeg
instance Extension NonPeriodicFixedPriceLeg Leg
 
-- | The physical delivery conditions for an oil product. 
data OilDelivery
instance Eq OilDelivery
instance Show OilDelivery
instance SchemaType OilDelivery
 
-- | Physically settled leg of a physically settled oil product 
--   transaction. 
data OilPhysicalLeg
instance Eq OilPhysicalLeg
instance Show OilPhysicalLeg
instance SchemaType OilPhysicalLeg
instance Extension OilPhysicalLeg PhysicalSwapLeg
instance Extension OilPhysicalLeg CommoditySwapLeg
instance Extension OilPhysicalLeg Leg
 
-- | The physical delivery conditions specific to an oil product 
--   delivered by pipeline. 
data OilPipelineDelivery
instance Eq OilPipelineDelivery
instance Show OilPipelineDelivery
instance SchemaType OilPipelineDelivery
 
-- | The specification of the oil product to be delivered. 
data OilProduct
instance Eq OilProduct
instance Show OilProduct
instance SchemaType OilProduct
 
-- | The type of physical commodity product to be delivered. 
data OilProductType
data OilProductTypeAttributes
instance Eq OilProductType
instance Eq OilProductTypeAttributes
instance Show OilProductType
instance Show OilProductTypeAttributes
instance SchemaType OilProductType
instance Extension OilProductType Scheme
 
-- | The physical delivery conditions specific to an oil product 
--   delivered by title transfer. 
data OilTransferDelivery
instance Eq OilTransferDelivery
instance Show OilTransferDelivery
instance SchemaType OilTransferDelivery
 
-- | The acceptable tolerance in the delivered quantity of a 
--   physical commodity product in terms of a percentage of the 
--   agreed delivery quantity. 
data PercentageTolerance
instance Eq PercentageTolerance
instance Show PercentageTolerance
instance SchemaType PercentageTolerance
 
-- | The common components of a physically settled leg of a 
--   Commodity Forward. This is an abstract type and should be 
--   extended by commodity-specific types. 
data PhysicalForwardLeg
instance Eq PhysicalForwardLeg
instance Show PhysicalForwardLeg
instance SchemaType PhysicalForwardLeg
instance Extension PhysicalForwardLeg CommodityForwardLeg
 
-- | The common components of a physically settled leg of a 
--   Commodity Swap. This is an abstract type and should be 
--   extended by commodity-specific types. 
data PhysicalSwapLeg
instance Eq PhysicalSwapLeg
instance Show PhysicalSwapLeg
instance SchemaType PhysicalSwapLeg
instance Extension PhysicalSwapLeg CommoditySwapLeg
 
-- | A pointer tyle reference to a Quantity schedule defined 
--   elsewhere. 
data QuantityScheduleReference
instance Eq QuantityScheduleReference
instance Show QuantityScheduleReference
instance SchemaType QuantityScheduleReference
instance Extension QuantityScheduleReference Reference
 
-- | A pointer tyle reference to a Quantity defined elsewhere. 
data QuantityReference
instance Eq QuantityReference
instance Show QuantityReference
instance SchemaType QuantityReference
instance Extension QuantityReference Reference
 
-- | A Disruption Fallback with the sequence in which it should 
--   be applied relative to other Disruption Fallbacks. 
data SequencedDisruptionFallback
instance Eq SequencedDisruptionFallback
instance Show SequencedDisruptionFallback
instance SchemaType SequencedDisruptionFallback
 
-- | Specifies a set of Settlement Periods associated with an 
--   Electricity Transaction for delivery on an Applicable Day 
--   or for a series of Applicable Days. 
data SettlementPeriods
instance Eq SettlementPeriods
instance Show SettlementPeriods
instance SchemaType SettlementPeriods
 
-- | A type defining the Fixed Price applicable to a range or 
--   ranges of Settlement Periods. 
data SettlementPeriodsFixedPrice
instance Eq SettlementPeriodsFixedPrice
instance Show SettlementPeriodsFixedPrice
instance SchemaType SettlementPeriodsFixedPrice
instance Extension SettlementPeriodsFixedPrice FixedPrice
 
-- | Allows a set of Settlement Periods to reference one already 
--   defined elsewhere in the trade. 
data SettlementPeriodsReference
instance Eq SettlementPeriodsReference
instance Show SettlementPeriodsReference
instance SchemaType SettlementPeriodsReference
instance Extension SettlementPeriodsReference Reference
 
-- | The specification of the Settlement Periods in which the 
--   electricity will be delivered for a "shaped" trade i.e. 
--   where different Settlement Period ranges will apply to 
--   different periods of the trade. 
data SettlementPeriodsSchedule
instance Eq SettlementPeriodsSchedule
instance Show SettlementPeriodsSchedule
instance SchemaType SettlementPeriodsSchedule
 
-- | A reference to the range of Settlement Periods that applies 
--   to a given period of a transaction. 
data SettlementPeriodsStep
instance Eq SettlementPeriodsStep
instance Show SettlementPeriodsStep
instance SchemaType SettlementPeriodsStep
 
-- | A quantity and associated unit. 
data UnitQuantity
instance Eq UnitQuantity
instance Show UnitQuantity
instance SchemaType UnitQuantity
 
-- | The physical leg of a Commodity Forward Transaction for 
--   which the underlyer is Bullion. 
elementBullionPhysicalLeg :: XMLParser BullionPhysicalLeg
elementToXMLBullionPhysicalLeg :: BullionPhysicalLeg -> [Content ()]
 
-- | Physically settled coal leg. 
elementCoalPhysicalLeg :: XMLParser CoalPhysicalLeg
elementToXMLCoalPhysicalLeg :: CoalPhysicalLeg -> [Content ()]
 
-- | Defines a commodity forward product. 
elementCommodityForward :: XMLParser CommodityForward
elementToXMLCommodityForward :: CommodityForward -> [Content ()]
 
-- | Defines the substitutable commodity forward leg 
elementCommodityForwardLeg :: XMLParser CommodityForwardLeg
 
-- | Defines a commodity option product. 
elementCommodityOption :: XMLParser CommodityOption
elementToXMLCommodityOption :: CommodityOption -> [Content ()]
 
-- | Defines a commodity swap product. 
elementCommoditySwap :: XMLParser CommoditySwap
elementToXMLCommoditySwap :: CommoditySwap -> [Content ()]
 
-- | Defines a commodity swaption product 
elementCommoditySwaption :: XMLParser CommoditySwaption
elementToXMLCommoditySwaption :: CommoditySwaption -> [Content ()]
 
-- | Defines the substitutable commodity swap leg 
elementCommoditySwapLeg :: XMLParser CommoditySwapLeg
 
-- | Physically settled electricity leg. 
elementElectricityPhysicalLeg :: XMLParser ElectricityPhysicalLeg
elementToXMLElectricityPhysicalLeg :: ElectricityPhysicalLeg -> [Content ()]
 
-- | Fixed Price Leg. 
elementFixedLeg :: XMLParser FixedPriceLeg
elementToXMLFixedLeg :: FixedPriceLeg -> [Content ()]
 
-- | Floating Price leg. 
elementFloatingLeg :: XMLParser FloatingPriceLeg
elementToXMLFloatingLeg :: FloatingPriceLeg -> [Content ()]
 
-- | Physically settled natural gas leg. 
elementGasPhysicalLeg :: XMLParser GasPhysicalLeg
elementToXMLGasPhysicalLeg :: GasPhysicalLeg -> [Content ()]
 
-- | Physically settled oil or refined products leg. 
elementOilPhysicalLeg :: XMLParser OilPhysicalLeg
elementToXMLOilPhysicalLeg :: OilPhysicalLeg -> [Content ()]
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
