{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Events.Business
  ( module Data.FpML.V53.Events.Business
  , module Data.FpML.V53.Msg
  , module Data.FpML.V53.Asset
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Msg
import {-# SOURCE #-} Data.FpML.V53.Asset
 
-- | A type defining an event identifier issued by the indicated 
--   party. 
data BusinessEventIdentifier
instance Eq BusinessEventIdentifier
instance Show BusinessEventIdentifier
instance SchemaType BusinessEventIdentifier
 
-- | A post-trade event reference identifier allocated by a 
--   party. FpML does not define the domain values associated 
--   with this element. Note that the domain values for this 
--   element are not strictly an enumerated list. 
data EventId
data EventIdAttributes
instance Eq EventId
instance Eq EventIdAttributes
instance Show EventId
instance Show EventIdAttributes
instance SchemaType EventId
instance Extension EventId Scheme
 
-- | Abstract base type for all events. 
data AbstractEvent
instance Eq AbstractEvent
instance Show AbstractEvent
instance SchemaType AbstractEvent
 
-- | Abstract base type for an extension/substitution point to 
--   customize FpML and add additional events. 
data AdditionalEvent
instance Eq AdditionalEvent
instance Show AdditionalEvent
instance SchemaType AdditionalEvent
instance Extension AdditionalEvent AbstractEvent
 
-- | Abstract base type for non-negotiated trade change 
--   descriptions 
data ChangeEvent
instance Eq ChangeEvent
instance Show ChangeEvent
instance SchemaType ChangeEvent
instance Extension ChangeEvent AbstractEvent
 
-- | A type that shows how multiple trades have been combined 
--   into a result. 
data CompressionActivity
instance Eq CompressionActivity
instance Show CompressionActivity
instance SchemaType CompressionActivity
 
-- | A type that identifies the type of trade amalgamation, for 
--   example netting or portfolio compression. 
data CompressionType
data CompressionTypeAttributes
instance Eq CompressionType
instance Eq CompressionTypeAttributes
instance Show CompressionType
instance Show CompressionTypeAttributes
instance SchemaType CompressionType
instance Extension CompressionType Scheme
 
-- | A structure describing an de-clear event. 
data DeClear
instance Eq DeClear
instance Show DeClear
instance SchemaType DeClear
 
-- | A structure describing the removal of a trade from a 
--   service, such as a reporting service. 
data Withdrawal
instance Eq Withdrawal
instance Show Withdrawal
instance SchemaType Withdrawal
 
-- | A type that describes what the requester would like to see 
--   done to implement the withdrawal, e.g. ExpungeRecords, 
--   RetainRecords. 
data RequestedWithdrawalAction
data RequestedWithdrawalActionAttributes
instance Eq RequestedWithdrawalAction
instance Eq RequestedWithdrawalActionAttributes
instance Show RequestedWithdrawalAction
instance Show RequestedWithdrawalActionAttributes
instance SchemaType RequestedWithdrawalAction
instance Extension RequestedWithdrawalAction Scheme
 
-- | A type that describes why a trade was withdrawn. 
data WithdrawalReason
data WithdrawalReasonAttributes
instance Eq WithdrawalReason
instance Eq WithdrawalReasonAttributes
instance Show WithdrawalReason
instance Show WithdrawalReasonAttributes
instance SchemaType WithdrawalReason
instance Extension WithdrawalReason Scheme
 
-- | A structure that describes a proposed match between trades 
--   or post-trade event reports. 
data EventProposedMatch
instance Eq EventProposedMatch
instance Show EventProposedMatch
instance SchemaType EventProposedMatch
 
data EventsChoice
instance Eq EventsChoice
instance Show EventsChoice
instance SchemaType EventsChoice
 
-- | A structure describing the effect of a change to an index. 
data IndexChange
instance Eq IndexChange
instance Show IndexChange
instance SchemaType IndexChange
instance Extension IndexChange ChangeEvent
instance Extension IndexChange AbstractEvent
 
-- | A structure describing an option exercise. 
data OptionExercise
instance Eq OptionExercise
instance Show OptionExercise
instance SchemaType OptionExercise
instance Extension OptionExercise AbstractEvent
 
-- | A structure describing an option expiring (i.e. passing its 
--   last exercise time and becoming worthless.) 
data OptionExpiry
instance Eq OptionExpiry
instance Show OptionExpiry
instance SchemaType OptionExpiry
instance Extension OptionExpiry AbstractEvent
 
-- | A structure describing an option expiring. 
data OptionExpiryBase
instance Eq OptionExpiryBase
instance Show OptionExpiryBase
instance SchemaType OptionExpiryBase
 
-- | A structure that describes how an option settles into a 
--   physical trade. 
data PhysicalSettlement
instance Eq PhysicalSettlement
instance Show PhysicalSettlement
instance SchemaType PhysicalSettlement
 
data PhysicalExercise
instance Eq PhysicalExercise
instance Show PhysicalExercise
instance SchemaType PhysicalExercise
 
-- | A type that describes why a trade terminated. 
data TerminatingEvent
data TerminatingEventAttributes
instance Eq TerminatingEvent
instance Eq TerminatingEventAttributes
instance Show TerminatingEvent
instance Show TerminatingEventAttributes
instance SchemaType TerminatingEvent
instance Extension TerminatingEvent Scheme
 
-- | A structure describing a negotiated amendment. 
data TradeAmendmentContent
instance Eq TradeAmendmentContent
instance Show TradeAmendmentContent
instance SchemaType TradeAmendmentContent
instance Extension TradeAmendmentContent AbstractEvent
 
-- | A structure describing a trade change. 
data TradeChangeBase
instance Eq TradeChangeBase
instance Show TradeChangeBase
instance SchemaType TradeChangeBase
instance Extension TradeChangeBase AbstractEvent
 
-- | A structure describing a non-negotiated trade resulting 
--   from a market event. 
data TradeChangeContent
instance Eq TradeChangeContent
instance Show TradeChangeContent
instance SchemaType TradeChangeContent
 
-- | A structure describing a trade maturing. 
data TradeMaturity
instance Eq TradeMaturity
instance Show TradeMaturity
instance SchemaType TradeMaturity
 
-- | A structure describing a change to the trade notional. 
data TradeNotionalChange
instance Eq TradeNotionalChange
instance Show TradeNotionalChange
instance SchemaType TradeNotionalChange
instance Extension TradeNotionalChange TradeChangeBase
instance Extension TradeNotionalChange AbstractEvent
 
-- | A structure describing a novation. 
data TradeNovationContent
instance Eq TradeNovationContent
instance Show TradeNovationContent
instance SchemaType TradeNovationContent
instance Extension TradeNovationContent AbstractEvent
 
-- | Defines a type that allows trade identifiers and/or trade 
--   information to be represented for a trade. 
data TradeReferenceInformation
instance Eq TradeReferenceInformation
instance Show TradeReferenceInformation
instance SchemaType TradeReferenceInformation
 
-- | The additionalEvent element is an extension/substitution 
--   point to customize FpML and add additional events. 
elementAdditionalEvent :: XMLParser AdditionalEvent
 
-- | Abstract substitutable place holder for specific change 
--   details. 
elementChangeEvent :: XMLParser ChangeEvent
elementToXMLChangeEvent :: ChangeEvent -> [Content ()]
 
-- | Describes a change due to an index component being 
--   adjusted. 
elementIndexChange :: XMLParser IndexChange
elementToXMLIndexChange :: IndexChange -> [Content ()]
 
 
 
 
 
 
 
 
 
 
