{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Mktenv
  ( module Data.FpML.V53.Mktenv
  , module Data.FpML.V53.Doc
  , module Data.FpML.V53.Asset
  , module Data.FpML.V53.Riskdef
  , module Data.FpML.V53.CD
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Doc
import {-# SOURCE #-} Data.FpML.V53.Asset
import {-# SOURCE #-} Data.FpML.V53.Riskdef
import {-# SOURCE #-} Data.FpML.V53.CD
 
-- | The frequency at which a rate is compounded. 
data CompoundingFrequency
data CompoundingFrequencyAttributes
instance Eq CompoundingFrequency
instance Eq CompoundingFrequencyAttributes
instance Show CompoundingFrequency
instance Show CompoundingFrequencyAttributes
instance SchemaType CompoundingFrequency
instance Extension CompoundingFrequency Scheme
 
-- | A generic credit curve definition. 
data CreditCurve
instance Eq CreditCurve
instance Show CreditCurve
instance SchemaType CreditCurve
instance Extension CreditCurve PricingStructure
 
-- | A set of credit curve values, which can include pricing 
--   inputs (which are typically credit spreads), default 
--   probabilities, and recovery rates. 
data CreditCurveValuation
instance Eq CreditCurveValuation
instance Show CreditCurveValuation
instance SchemaType CreditCurveValuation
instance Extension CreditCurveValuation PricingStructureValuation
instance Extension CreditCurveValuation Valuation
 
-- | A set of default probabilities. 
data DefaultProbabilityCurve
instance Eq DefaultProbabilityCurve
instance Show DefaultProbabilityCurve
instance SchemaType DefaultProbabilityCurve
instance Extension DefaultProbabilityCurve PricingStructureValuation
instance Extension DefaultProbabilityCurve Valuation
 
-- | A curve used to model a set of forward interest rates. Used 
--   for forecasting interest rates as part of a pricing 
--   calculation. 
data ForwardRateCurve
instance Eq ForwardRateCurve
instance Show ForwardRateCurve
instance SchemaType ForwardRateCurve
 
-- | An fx curve object., which includes pricing inputs and term 
--   structures for fx forwards. 
data FxCurve
instance Eq FxCurve
instance Show FxCurve
instance SchemaType FxCurve
instance Extension FxCurve PricingStructure
 
-- | A valuation of an FX curve object., which includes pricing 
--   inputs and term structures for fx forwards. 
data FxCurveValuation
instance Eq FxCurveValuation
instance Show FxCurveValuation
instance SchemaType FxCurveValuation
instance Extension FxCurveValuation PricingStructureValuation
instance Extension FxCurveValuation Valuation
 
-- | A collection of spot FX rates used in pricing. 
data FxRateSet
instance Eq FxRateSet
instance Show FxRateSet
instance SchemaType FxRateSet
instance Extension FxRateSet QuotedAssetSet
 
-- | A pricing data set that contains a series of points with 
--   coordinates. It is a sparse matrix representation of a 
--   multi-dimensional matrix. 
data MultiDimensionalPricingData
instance Eq MultiDimensionalPricingData
instance Show MultiDimensionalPricingData
instance SchemaType MultiDimensionalPricingData
 
-- | An adjustment used to accommodate a parameter of the input 
--   trade, e.g. the strike. 
data ParametricAdjustment
instance Eq ParametricAdjustment
instance Show ParametricAdjustment
instance SchemaType ParametricAdjustment
 
-- | A value of the adjustment point, consisting of the x value 
--   and the corresponding y value. 
data ParametricAdjustmentPoint
instance Eq ParametricAdjustmentPoint
instance Show ParametricAdjustmentPoint
instance SchemaType ParametricAdjustmentPoint
 
-- | A single valued point with a set of coordinates that define 
--   an arbitrary number of indentifying indexes (0 or more). 
--   Note that the collection of coordinates/coordinate 
--   references for a PricingStructurePoint must not define a 
--   given dimension (other than "generic") more than once. This 
--   is to avoid ambiguity. 
data PricingStructurePoint
instance Eq PricingStructurePoint
instance Show PricingStructurePoint
instance SchemaType PricingStructurePoint
 
-- | A curve consisting only of values over a term. This is a 
--   restricted form of One Dimensional Structure. 
data TermCurve
instance Eq TermCurve
instance Show TermCurve
instance SchemaType TermCurve
 
-- | A value point that can have a time dimension. Allows bid, 
--   mid, ask, and spread values to be represented. 
data TermPoint
instance Eq TermPoint
instance Show TermPoint
instance SchemaType TermPoint
 
-- | A matrix of volatilities with dimension 0-3. 
data VolatilityMatrix
instance Eq VolatilityMatrix
instance Show VolatilityMatrix
instance SchemaType VolatilityMatrix
instance Extension VolatilityMatrix PricingStructureValuation
instance Extension VolatilityMatrix Valuation
 
-- | A representation of volatilities of an asset. This is a 
--   generic structure whose values can be supplied in a 
--   specific volatility matrix. 
data VolatilityRepresentation
instance Eq VolatilityRepresentation
instance Show VolatilityRepresentation
instance SchemaType VolatilityRepresentation
instance Extension VolatilityRepresentation PricingStructure
 
-- | A generic yield curve object, which can be valued in a 
--   variety of ways. 
data YieldCurve
instance Eq YieldCurve
instance Show YieldCurve
instance SchemaType YieldCurve
instance Extension YieldCurve PricingStructure
 
-- | The values of a yield curve, including possibly inputs and 
--   outputs (dfs, forwards, zero rates). 
data YieldCurveValuation
instance Eq YieldCurveValuation
instance Show YieldCurveValuation
instance SchemaType YieldCurveValuation
instance Extension YieldCurveValuation PricingStructureValuation
instance Extension YieldCurveValuation Valuation
 
-- | A curve used to model a set of zero-coupon interest rates. 
data ZeroRateCurve
instance Eq ZeroRateCurve
instance Show ZeroRateCurve
instance SchemaType ZeroRateCurve
 
elementCreditCurve :: XMLParser CreditCurve
elementToXMLCreditCurve :: CreditCurve -> [Content ()]
 
elementCreditCurveValuation :: XMLParser CreditCurveValuation
elementToXMLCreditCurveValuation :: CreditCurveValuation -> [Content ()]
 
elementFxCurve :: XMLParser FxCurve
elementToXMLFxCurve :: FxCurve -> [Content ()]
 
elementFxCurveValuation :: XMLParser FxCurveValuation
elementToXMLFxCurveValuation :: FxCurveValuation -> [Content ()]
 
elementVolatilityMatrixValuation :: XMLParser VolatilityMatrix
elementToXMLVolatilityMatrixValuation :: VolatilityMatrix -> [Content ()]
 
elementVolatilityRepresentation :: XMLParser VolatilityRepresentation
elementToXMLVolatilityRepresentation :: VolatilityRepresentation -> [Content ()]
 
elementYieldCurve :: XMLParser YieldCurve
elementToXMLYieldCurve :: YieldCurve -> [Content ()]
 
elementYieldCurveValuation :: XMLParser YieldCurveValuation
elementToXMLYieldCurveValuation :: YieldCurveValuation -> [Content ()]
 
 
 
 
 
 
