{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Notification.CreditEvent
  ( module Data.FpML.V53.Notification.CreditEvent
  , module Data.FpML.V53.Msg
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Msg
 
data AffectedTransactions
instance Eq AffectedTransactions
instance Show AffectedTransactions
instance SchemaType AffectedTransactions
 
data BankruptcyEvent
instance Eq BankruptcyEvent
instance Show BankruptcyEvent
instance SchemaType BankruptcyEvent
instance Extension BankruptcyEvent CreditEvent
 
data CreditEvent
instance Eq CreditEvent
instance Show CreditEvent
instance SchemaType CreditEvent
 
-- | An event type that records the occurrence of a credit event 
--   notice. 
data CreditEventNoticeDocument
instance Eq CreditEventNoticeDocument
instance Show CreditEventNoticeDocument
instance SchemaType CreditEventNoticeDocument
 
-- | A message type defining the ISDA defined Credit Event 
--   Notice. ISDA defines it as an irrevocable notice from a 
--   Notifying Party to the other party that describes a Credit 
--   Event that occurred. A Credit Event Notice must contain 
--   detail of the facts relevant to the determination that a 
--   Credit Event has occurred. 
data CreditEventNotification
instance Eq CreditEventNotification
instance Show CreditEventNotification
instance SchemaType CreditEventNotification
instance Extension CreditEventNotification CorrectableRequestMessage
instance Extension CreditEventNotification RequestMessage
instance Extension CreditEventNotification Message
instance Extension CreditEventNotification Document
 
data FailureToPayEvent
instance Eq FailureToPayEvent
instance Show FailureToPayEvent
instance SchemaType FailureToPayEvent
instance Extension FailureToPayEvent CreditEvent
 
data ObligationAccelerationEvent
instance Eq ObligationAccelerationEvent
instance Show ObligationAccelerationEvent
instance SchemaType ObligationAccelerationEvent
instance Extension ObligationAccelerationEvent CreditEvent
 
data ObligationDefaultEvent
instance Eq ObligationDefaultEvent
instance Show ObligationDefaultEvent
instance SchemaType ObligationDefaultEvent
instance Extension ObligationDefaultEvent CreditEvent
 
data RepudiationMoratoriumEvent
instance Eq RepudiationMoratoriumEvent
instance Show RepudiationMoratoriumEvent
instance SchemaType RepudiationMoratoriumEvent
instance Extension RepudiationMoratoriumEvent CreditEvent
 
data RestructuringEvent
instance Eq RestructuringEvent
instance Show RestructuringEvent
instance SchemaType RestructuringEvent
instance Extension RestructuringEvent CreditEvent
 
elementBankruptcy :: XMLParser BankruptcyEvent
elementToXMLBankruptcy :: BankruptcyEvent -> [Content ()]
 
elementCreditEvent :: XMLParser CreditEvent
 
-- | A global element used to hold CENs. 
elementCreditEventNotice :: XMLParser CreditEventNoticeDocument
elementToXMLCreditEventNotice :: CreditEventNoticeDocument -> [Content ()]
 
elementFailureToPay :: XMLParser FailureToPayEvent
elementToXMLFailureToPay :: FailureToPayEvent -> [Content ()]
 
elementObligationAcceleration :: XMLParser ObligationAccelerationEvent
elementToXMLObligationAcceleration :: ObligationAccelerationEvent -> [Content ()]
 
elementObligationDefault :: XMLParser ObligationDefaultEvent
elementToXMLObligationDefault :: ObligationDefaultEvent -> [Content ()]
 
elementRepudiationMoratorium :: XMLParser RepudiationMoratoriumEvent
elementToXMLRepudiationMoratorium :: RepudiationMoratoriumEvent -> [Content ()]
 
elementRestructuring :: XMLParser RestructuringEvent
elementToXMLRestructuring :: RestructuringEvent -> [Content ()]
 
-- | Credit Event Notification message. 
 
-- | A message defining the ISDA defined Credit Event Notice. 
--   ISDA defines it as an irrevocable notice from a Notifying 
--   Party to the other party that describes a Credit Event that 
--   occurred. A Credit Event Notice must contain detail of the 
--   facts relevant to the determination that a Credit Event has 
--   occurred. 
elementCreditEventNotification :: XMLParser CreditEventNotification
elementToXMLCreditEventNotification :: CreditEventNotification -> [Content ()]
 
elementCreditEventAcknowledgement :: XMLParser Acknowledgement
elementToXMLCreditEventAcknowledgement :: Acknowledgement -> [Content ()]
 
elementCreditEventException :: XMLParser Exception
elementToXMLCreditEventException :: Exception -> [Content ()]
