{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Reporting.Valuation
  ( module Data.FpML.V53.Reporting.Valuation
  , module Data.FpML.V53.Events.Business
  , module Data.FpML.V53.Valuation
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Events.Business
import {-# SOURCE #-} Data.FpML.V53.Valuation
 
-- | A type used to describe the scope/contents of a report. 
data ReportContents
instance Eq ReportContents
instance Show ReportContents
instance SchemaType ReportContents
 
-- | A type used in valuation enquiry messages which relates a 
--   portfolio to its trades and current value. 
data PortfolioValuationItem
instance Eq PortfolioValuationItem
instance Show PortfolioValuationItem
instance SchemaType PortfolioValuationItem
 
-- | A type defining the content model for a message allowing 
--   one party a report containing valuations of one or many 
--   existing trades. 
data RequestValuationReport
instance Eq RequestValuationReport
instance Show RequestValuationReport
instance SchemaType RequestValuationReport
instance Extension RequestValuationReport CorrectableRequestMessage
instance Extension RequestValuationReport RequestMessage
instance Extension RequestValuationReport Message
instance Extension RequestValuationReport Document
 
-- | A type used in trade valuation enquiry messages which 
--   relates a trade identifier to its current value. 
data TradeValuationItem
instance Eq TradeValuationItem
instance Show TradeValuationItem
instance SchemaType TradeValuationItem
 
-- | A type defining the content model for a message normally 
--   generated in response to a RequestValuationReport request. 
data ValuationReport
instance Eq ValuationReport
instance Show ValuationReport
instance SchemaType ValuationReport
instance Extension ValuationReport NotificationMessage
instance Extension ValuationReport Message
instance Extension ValuationReport Document
 
-- | A type defining the content model for a message that 
--   retracts a valuation report. This says that the most 
--   recently supplied valuation is erroneous and a previous 
--   value should be used. 
data ValuationReportRetracted
instance Eq ValuationReportRetracted
instance Show ValuationReportRetracted
instance SchemaType ValuationReportRetracted
instance Extension ValuationReportRetracted NotificationMessage
instance Extension ValuationReportRetracted Message
instance Extension ValuationReportRetracted Document
 
-- | Global portfolio element used as a basis for a substitution 
--   group. 
elementPortfolio :: XMLParser Portfolio
elementToXMLPortfolio :: Portfolio -> [Content ()]
 
-- | Global element used to substitute for "portfolio". 
elementQueryPortfolio :: XMLParser QueryPortfolio
elementToXMLQueryPortfolio :: QueryPortfolio -> [Content ()]
 
-- | Reporting messages. 
 
elementRequestValuationReport :: XMLParser RequestValuationReport
elementToXMLRequestValuationReport :: RequestValuationReport -> [Content ()]
 
elementValuationReport :: XMLParser ValuationReport
elementToXMLValuationReport :: ValuationReport -> [Content ()]
 
elementValuationReportRetracted :: XMLParser ValuationReportRetracted
elementToXMLValuationReportRetracted :: ValuationReportRetracted -> [Content ()]
 
elementValuationReportAcknowledgement :: XMLParser Acknowledgement
elementToXMLValuationReportAcknowledgement :: Acknowledgement -> [Content ()]
 
elementValuationReportException :: XMLParser Exception
elementToXMLValuationReportException :: Exception -> [Content ()]
