{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Riskdef
  ( module Data.FpML.V53.Riskdef
  , module Data.FpML.V53.Doc
  , module Data.FpML.V53.Asset
--  , module Data.FpML.V53.Mktenv
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Doc
import {-# SOURCE #-} Data.FpML.V53.Asset
--import {-# SOURCE #-} Data.FpML.V53.Mktenv
 
-- | Reference to an underlying asset, term point or pricing 
--   structure (yield curve). 
data AssetOrTermPointOrPricingStructureReference
instance Eq AssetOrTermPointOrPricingStructureReference
instance Show AssetOrTermPointOrPricingStructureReference
instance SchemaType AssetOrTermPointOrPricingStructureReference
instance Extension AssetOrTermPointOrPricingStructureReference Reference
 
-- | A structure that holds a set of measures about an asset. 
data BasicAssetValuation
instance Eq BasicAssetValuation
instance Show BasicAssetValuation
instance SchemaType BasicAssetValuation
instance Extension BasicAssetValuation Valuation
 
-- | The type defining a denominator term of the formula. Its 
--   value is (sum of weighted partials) ^ power. 
data DenominatorTerm
instance Eq DenominatorTerm
instance Show DenominatorTerm
instance SchemaType DenominatorTerm
 
-- | The method by which a derivative is computed. 
data DerivativeCalculationMethod
data DerivativeCalculationMethodAttributes
instance Eq DerivativeCalculationMethod
instance Eq DerivativeCalculationMethodAttributes
instance Show DerivativeCalculationMethod
instance Show DerivativeCalculationMethodAttributes
instance SchemaType DerivativeCalculationMethod
instance Extension DerivativeCalculationMethod Scheme
 
-- | A description of how a numerical derivative is computed. 
data DerivativeCalculationProcedure
instance Eq DerivativeCalculationProcedure
instance Show DerivativeCalculationProcedure
instance SchemaType DerivativeCalculationProcedure
 
-- | A formula for computing a complex derivative from partial 
--   derivatives. Its value is the sum of the terms divided by 
--   the product of the denominator terms. 
data DerivativeFormula
instance Eq DerivativeFormula
instance Show DerivativeFormula
instance SchemaType DerivativeFormula
 
-- | A type defining a term of the formula. Its value is the 
--   product of the its coefficient and the referenced partial 
--   derivatives. 
data FormulaTerm
instance Eq FormulaTerm
instance Show FormulaTerm
instance SchemaType FormulaTerm
 
-- | A generic (user defined) dimension, e.g. for use in a 
--   correlation surface. e.g. a currency, stock, etc. This 
--   would take values like USD, GBP, JPY, or IBM, MSFT, etc. 
data GenericDimension
data GenericDimensionAttributes
instance Eq GenericDimension
instance Eq GenericDimensionAttributes
instance Show GenericDimension
instance Show GenericDimensionAttributes
instance SchemaType GenericDimension
instance Extension GenericDimension Xsd.XsdString
 
-- | A collection of instruments usable for quotation purposes. 
--   In future releases, quotable derivative assets may be added 
--   after the underlying asset. 
data InstrumentSet
instance Eq InstrumentSet
instance Show InstrumentSet
instance SchemaType InstrumentSet
 
-- | A collection of pricing inputs. 
data Market
instance Eq Market
instance Show Market
instance SchemaType Market
 
-- | Reference to a market structure. 
data MarketReference
instance Eq MarketReference
instance Show MarketReference
instance SchemaType MarketReference
instance Extension MarketReference Reference
 
-- | The type of perturbation applied to compute a derivative 
--   perturbatively. 
data PerturbationType
data PerturbationTypeAttributes
instance Eq PerturbationType
instance Eq PerturbationTypeAttributes
instance Show PerturbationType
instance Show PerturbationTypeAttributes
instance SchemaType PerturbationType
instance Extension PerturbationType Scheme
 
-- | A unique identifier for the position. The id attribute is 
--   defined for intradocument referencing. 
data PositionId
data PositionIdAttributes
instance Eq PositionId
instance Eq PositionIdAttributes
instance Show PositionId
instance Show PositionIdAttributes
instance SchemaType PositionId
instance Extension PositionId Scheme
 
-- | The substitution of a pricing input (e.g. curve) for 
--   another, used in generating prices and risks for valuation 
--   scenarios. 
data PricingInputReplacement
instance Eq PricingInputReplacement
instance Show PricingInputReplacement
instance SchemaType PricingInputReplacement
 
-- | The type of pricing structure represented. 
data PricingInputType
data PricingInputTypeAttributes
instance Eq PricingInputType
instance Eq PricingInputTypeAttributes
instance Show PricingInputType
instance Show PricingInputTypeAttributes
instance SchemaType PricingInputType
instance Extension PricingInputType Scheme
 
-- | A set of index values that identify a pricing data point. 
--   For example: (strike = 17%, expiration = 6M, term = 1Y. 
data PricingDataPointCoordinate
instance Eq PricingDataPointCoordinate
instance Show PricingDataPointCoordinate
instance SchemaType PricingDataPointCoordinate
 
-- | Reference to a Pricing Data Point Coordinate. 
data PricingDataPointCoordinateReference
instance Eq PricingDataPointCoordinateReference
instance Show PricingDataPointCoordinateReference
instance SchemaType PricingDataPointCoordinateReference
instance Extension PricingDataPointCoordinateReference Reference
 
-- | For an asset (e.g. a reference/benchmark asset), the 
--   pricing structure used to price it. Used, for example, to 
--   specify that the rateIndex "USD-LIBOR-Telerate" with term = 
--   6M is priced using the "USD-LIBOR-Close" curve. 
data PricingMethod
instance Eq PricingMethod
instance Show PricingMethod
instance SchemaType PricingMethod
 
-- | A definition of the mathematical derivative with respect to 
--   a specific pricing parameter. 
data PricingParameterDerivative
instance Eq PricingParameterDerivative
instance Show PricingParameterDerivative
instance SchemaType PricingParameterDerivative
 
-- | Reference to a partial derivative. 
data PricingParameterDerivativeReference
instance Eq PricingParameterDerivativeReference
instance Show PricingParameterDerivativeReference
instance SchemaType PricingParameterDerivativeReference
instance Extension PricingParameterDerivativeReference Reference
 
-- | A definition of a shift with respect to a specific pricing 
--   parameter. 
data PricingParameterShift
instance Eq PricingParameterShift
instance Show PricingParameterShift
instance SchemaType PricingParameterShift
 
-- | An abstract pricing structure valuation base type. Used as 
--   a base for values of pricing structures such as yield 
--   curves and volatility matrices. Derived from the 
--   "Valuation" type. 
data PricingStructureValuation
instance Eq PricingStructureValuation
instance Show PricingStructureValuation
instance SchemaType PricingStructureValuation
instance Extension PricingStructureValuation Valuation
 
-- | A collection of quoted assets. 
data QuotedAssetSet
instance Eq QuotedAssetSet
instance Show QuotedAssetSet
instance SchemaType QuotedAssetSet
 
-- | A set of characteristics describing a sensitivity. 
data SensitivityDefinition
instance Eq SensitivityDefinition
instance Show SensitivityDefinition
instance SchemaType SensitivityDefinition
 
-- | A sensitivity report definition, consisting of a collection 
--   of sensitivity definitions. 
data SensitivitySetDefinition
instance Eq SensitivitySetDefinition
instance Show SensitivitySetDefinition
instance SchemaType SensitivitySetDefinition
 
-- | A reference to a sensitivity set definition. 
data SensitivitySetDefinitionReference
instance Eq SensitivitySetDefinitionReference
instance Show SensitivitySetDefinitionReference
instance SchemaType SensitivitySetDefinitionReference
instance Extension SensitivitySetDefinitionReference Reference
 
-- | The time dimensions of a term-structure. The user must 
--   supply either a tenor or a date or both. 
data TimeDimension
instance Eq TimeDimension
instance Show TimeDimension
instance SchemaType TimeDimension
 
-- | A valuation of an valuable object - an asset or a pricing 
--   input. This is an abstract type, used as a base for values 
--   of pricing structures such as yield curves as well as asset 
--   values. 
data Valuation
instance Eq Valuation
instance Show Valuation
instance SchemaType Valuation
 
-- | Reference to a Valuation or any derived structure such as 
--   PricingStructureValuation. 
data ValuationReference
instance Eq ValuationReference
instance Show ValuationReference
instance SchemaType ValuationReference
instance Extension ValuationReference Reference
 
-- | A set of rules for generating a valuation. 
data ValuationScenario
instance Eq ValuationScenario
instance Show ValuationScenario
instance SchemaType ValuationScenario
 
-- | Reference to a valuation scenario. 
data ValuationScenarioReference
instance Eq ValuationScenarioReference
instance Show ValuationScenarioReference
instance SchemaType ValuationScenarioReference
instance Extension ValuationScenarioReference Reference
 
-- | A partial derivative multiplied by a weighting factor. 
data WeightedPartialDerivative
instance Eq WeightedPartialDerivative
instance Show WeightedPartialDerivative
instance SchemaType WeightedPartialDerivative
 
-- | This is a global element used for creating global types. It 
--   holds Market information, e.g. curves, surfaces, quotes, 
--   etc. 
elementMarket :: XMLParser Market
elementToXMLMarket :: Market -> [Content ()]
 
elementPricingStructure :: XMLParser PricingStructure
elementToXMLPricingStructure :: PricingStructure -> [Content ()]
 
elementPricingStructureValuation :: XMLParser PricingStructureValuation
elementToXMLPricingStructureValuation :: PricingStructureValuation -> [Content ()]
 
 
 
 
 
 
 
 
 
 
